==> isddc-1.11.1-py3-none-any.whl: isddc/isddc_filesystem.py <==
import os
import json
import pickle

try:
    import yaml
except ModuleNotFoundError:
    pass

from . import idict


class FileDataOperator:

    def __init__(self, db_dir: str):
        self.root: str = gendir(db_dir)
        self.path: idict = fetch_deep_path(self.root)

    def __getitem__(self, file):
        try:
            full = self.path[file]
        except KeyError:
            raise FileNotFoundError(f'{abspath(self.root, file)}')

        try:
            if isfile(full):
                return Filetor(full)
            return os.listdir(full)
        except FileNotFoundError:
            raise FileNotFoundError(f'File or directory been deleted: {full}')

    def __setitem__(self, file, data):
        if file not in self.path:
            self.path[file] = abspath(self.root, file)

        full = self.path[file]

        if not isdir(dirname(full)):
            os.makedirs(dirname(full))

        Filetor(full, data)

    def __delitem__(self, file):
        try:
            full = self.path.pop(file)
        except KeyError:
            raise FileNotFoundError(
                f'File or directory does not exist: {abspath(self.root, file)}')

        if isfile(full):
            os.remove(full)
        else:
            os.removedirs(full)

    def __getattr__(self, file):
        return self[file]

    def __setattr__(self, file, value):
        if file in ('root', 'path'):
            return super().__setattr__(file, value)
        self[file] = value

    def __delattr__(self, file):
        del self[file]

    def __getstate__(self):
        return self.root, self.path

    def __setstate__(self, state):
        self.root, self.path = state

    def __iter__(self):
        yield from self.path

    def __str__(self):
        return f'{type(self).__name__}({self.root})'


class Filetor:
    """
    if `data` is None:
        Read `file` according to `ftype`.
    else:
        Write `data` to `file` according to `ftype`.
    """

    def __new__(
            cls,
            file: str,
            data: ... = None,
            *,
            ftype: 'enum(json, yaml, obj, txt)' = None,
            encoding: str = 'UTF-8'
    ):
        mode = 'r' if data is None else 'w'

        if ftype is None:
            ftype: str = file.rsplit('.', 1)[-1]

        if ftype == 'obj':
            mode += 'b'
            encoding = None

        with open(file, mode, encoding=encoding) as fp:
            try:
                return getattr(cls, f'_for_{ftype}_')(fp, data)
            except AttributeError:
                return cls._for_text_(fp, data)

    @classmethod
    def _for_json_(cls, fp: open, data=None):
        if data is None:
            return json.load(fp)
        return json.dump(data, fp)

    @classmethod
    def _for_yaml_(cls, fp: open, data=None):
        if data is None:
            return yaml.safe_load(fp)
        return yaml.safe_dump(data, fp)

    @classmethod
    def _for_obj_(cls, fp: open, obj=None):
        if obj is None:
            return pickle.load(fp)
        return pickle.dump(obj, fp)

    @classmethod
    def _for_text_(cls, fp: open, data=None):
        if data is None:
            return fp.read()
        return fp.write(str(data))

    _for_yml_ = _for_yaml_
    _for_txt_ = _for_text_


def fetch_deep_path(
        root: str = None,
        _paths: 'Not param' = None,
        _root: 'Not param' = None
) -> idict:
    paths = _paths or idict()
    _root = _root or root

    if root == _root:
        paths['root'] = root

    for name in os.listdir(root):
        full = abspath(root, name)
        paths[full.replace(_root, '')[1:].replace('\\', '/')] = full

        if isdir(full):
            fetch_deep_path(full, paths, _root)

    return paths


def abspath(*a) -> str:
    """Return an absolute path"""
    return os.path.abspath(os.path.join(*a))


def dirname(path: str, *, level: int = 1) -> str:
    """Culling directories from the
    end according to `level`, usually
    used to get the project root path.
    """
    for n in range(level):
        path = os.path.dirname(os.path.abspath(path))
    return path


def gendir(*a) -> str:
    """Generate path
    Generates a directory based on the passed in value,
    creates it if it does not exist, and returns it.
    """
    dir_: str = abspath(*a)
    exists(dir_) or os.makedirs(dir_)
    return dir_


isabs = os.path.isabs
isdir = os.path.isdir
isfile = os.path.isfile
exists = os.path.exists
basename = os.path.basename

==> isddc-3.1.0-py3-none-any.whl: isddc/isddc_filesystem.py <==
import os
import json
import pickle

try:
    import yaml
except ModuleNotFoundError:
    pass


class FileDataOperator:

    def __init__(self, db_dir: str):
        self.root: str = gendir(db_dir)
        self.path: dict = fetch_deep_path(self.root)

    def __getitem__(self, file):
        try:
            full = self.path[file]
        except KeyError:
            raise FileNotFoundError(f'{joinpath(self.root, file)}')

        try:
            if isfile(full):
                return Filetor(full)
            return os.listdir(full)
        except FileNotFoundError:
            raise FileNotFoundError(f'File or directory been deleted: {full}')

    def __setitem__(self, file, data):
        if file not in self.path:
            self.path[file] = joinpath(self.root, file)

        full = self.path[file]

        if not isdir(dirname(full)):
            os.makedirs(dirname(full))

        Filetor(full, data)

    def __delitem__(self, file):
        try:
            full = self.path.pop(file)
        except KeyError:
            raise FileNotFoundError(
                f'File or directory does not exist: {joinpath(self.root, file)}')

        if isfile(full):
            os.remove(full)
        else:
            os.removedirs(full)

    def __getattr__(self, file):
        return self[file]

    def __setattr__(self, file, value):
        if file in ('root', 'path'):
            return super().__setattr__(file, value)
        self[file] = value

    def __delattr__(self, file):
        del self[file]

    def __getstate__(self):
        return self.root, self.path

    def __setstate__(self, state):
        self.root, self.path = state

    def __iter__(self):
        yield from self.path

    def __str__(self):
        return f'{type(self).__name__}({self.root})'


class Filetor:
    """
    if `data` is None:
        Read `file` according to `ftype`.
    else:
        Write `data` to `file` according to `ftype`.
    """

    def __new__(
            cls,
            file: str,
            data: ... = None,
            *,
            ftype: 'enum(json, yaml, obj, txt)' = None,
            encoding: str = 'UTF-8',
            **kw
    ):
        mode = 'r' if data is None else 'w'

        if ftype is None:
            ftype: str = file.rsplit('.', 1)[-1]

        if ftype == 'obj':
            mode += 'b'
            encoding = None

        with open(file, mode, encoding=encoding) as fp:
            try:
                return getattr(cls, f'_for_{ftype}_')(fp, data, **kw)
            except AttributeError:
                return cls._for_text_(fp, data, **kw)

    @classmethod
    def _for_json_(cls, fp: open, data=None, **kw):
        if data is None:
            return json.load(fp, **kw)
        return json.dump(data, fp, **kw)

    @classmethod
    def _for_yaml_(cls, fp: open, data=None, **kw):
        if data is None:
            return yaml.safe_load(fp)
        return yaml.safe_dump(data, fp, **kw)

    @classmethod
    def _for_obj_(cls, fp: open, obj=None, **kw):
        if obj is None:
            return pickle.load(fp, **kw)
        return pickle.dump(obj, fp, **kw)

    @classmethod
    def _for_text_(cls, fp: open, data=None, **kw):
        if data is None:
            return fp.read(**kw)
        return fp.write(str(data))

    _for_yml_ = _for_yaml_
    _for_txt_ = _for_text_


def fetch_deep_path(
        path: str = None,
        _paths: 'Not param' = None,
        _root_path: 'Not param' = None
) -> dict:
    paths: dict = _paths or {}
    root_path: str = _root_path or abspath(path)

    if path == root_path:
        paths['_root'] = path

    for name in os.listdir(path):
        absolute_path: str = joinpath2abs(path, name)
        relative_path: str = absolute_path.replace(root_path, '')[1:].replace('\\', '/')
        paths[relative_path] = absolute_path

        if isdir(absolute_path):
            fetch_deep_path(absolute_path, paths, root_path)

    return paths


def joinpath2abs(*a) -> str:
    return abspath(joinpath(*a))


def dirname(path: str, *, level: int = 1) -> str:
    """Culling directories from the
    end according to `level`, usually
    used to get the project root path.
    """
    for n in range(level):
        path = os.path.dirname(abspath(path))
    return path


def gendir(*a) -> str:
    """Generate path
    Generates a directory based on the passed in value,
    creates it if it does not exist, and returns it.
    """
    dir_: str = joinpath2abs(*a)
    exists(dir_) or os.makedirs(dir_)
    return dir_


isabs = os.path.isabs
isdir = os.path.isdir
isfile = os.path.isfile
exists = os.path.exists
joinpath = os.path.join
abspath = os.path.abspath
basename = os.path.basename
